@@ -104,6 +104,12 @@ async def run_tcp_server(conn_handler: Callable, host: str, port: int):
 
 
 async def run_unix_server(conn_handler: Callable, path: str):
+    '''
+    `path`: a file path accessible by both the server and the client,
+        which run on the same machine. This is not a regular file,
+        and it (including parent directories) does not need to exist.
+        In fact, if the file exists, it will be removed first.
+    '''
     try:
         os.unlink(path)
     except FileNotFoundError:
@@ -133,7 +139,10 @@ async def open_tcp_connection(host, port, *, timeout=None):
             await asyncio.sleep(0.1)
 
 
-async def open_unix_connection(path, *, timeout=None):
+async def open_unix_connection(path: str, *, timeout=None):
+    '''
+    `path`: the same string that has been used by `run_unix_server`.
+    '''
     t0 = perf_counter()
     while True:
         try:
@@ -147,6 +156,24 @@ async def open_unix_connection(path, *, timeout=None):
 
 
 class SocketApplication(EnforceOverrides):
+    '''
+    SocketApplication is designed to to used similar to the "application" in a
+    HTTP framework. The main API is to register "endpoint" functions by the method
+    `add_route`. This allows to back the socket service by multiple functions
+    for different purposes. Usually there is only one main function, which involves
+    transmitting substantial amount of data between the server and the client.
+    For simplicity, one may use '/' for the `path` of this route.
+    The other routes are usually supportive, for example, getting server info
+    or setting options. For example
+
+        app.add_route('/', make_prediction)
+        app.add_route('/server-info', get_server_info)
+        app.add_route('/set-option', set_server_option)
+
+    This class is the intended interface between a socket server and a particular
+    application (functions). Usually, user should not customize the class
+    SocketServer.
+    '''
     def __init__(self, *,
                  on_startup: Sequence[Callable] = None,
                  on_shutdown: Sequence[Callable] = None,
@@ -158,25 +185,32 @@ class SocketApplication(EnforceOverrides):
     def add_route(self, path: str,
                   route: Union[Callable[[Any], Awaitable[Any]], Callable[[], Awaitable[Any]]]):
         '''
-        `route` is an async function that takes a single positional arg,
+        `route` is an *async* function that takes a single positional arg,
         and returns a response (which could be `None` if so desired).
         The response should be serializable by the encoder.
         To be safe, return a object of Python native types.
         If exception is raised in this method, appropriate `RemoteException`
         object will be sent in the response.
         The method could also proactively return a `RemoteException` object.
+
+        `path` is any string. The route is identified by this string. For familiarity,
+        it may be a good idea to start the string with '/', although this is in no
+        way necessary.
+
+        There is no GET/POST distinction like in the case of HTTP.
         '''
+        # TODO: support sync "endpoint" functions, if they are I/O bound.
         self._routes[path] = route
 
     async def handle_request(self, path: str, data: Any = None):
+        '''
+        Dispatch the request to a registered route function.
+        '''
         if data is None:
             return await self._routes[path]()
         return await self._routes[path](data)
 
 
-# Usually user should not customize this class.
-# Rather, they should design their worker class and "attach" it to
-# a SocketApplication instance.
 class SocketServer(EnforceOverrides):
     def __init__(self,
                  app: SocketApplication,
@@ -192,6 +226,9 @@ class SocketServer(EnforceOverrides):
             (Note, the client may open many connections.)
             This "concurrency" is in terms of concurrent calls to
             `handle_request`.
+
+        The type of the service, between 'tcp' and 'unix', is determined
+        by the parameters `path`, `host`, and `port`. See code for details.
         '''
         self.app = app
         if path:
@@ -211,7 +248,7 @@ class SocketServer(EnforceOverrides):
             self._host = host
             self._port = int(port)
         self._backlog = backlog or 64
-        self._encoder = 'orjson'  # encoder when sending responses
+        self._encoder = 'pickle'  # encoder when sending responses
         self._n_connections = 0
         self._shutdown_path = shutdown_path
         self.to_shutdown = False
@@ -225,6 +262,9 @@ class SocketServer(EnforceOverrides):
         return self.__repr__()
 
     async def run(self):
+        '''
+        Start the server and let it stay up until shutdown conditions are met.
+        '''
         if self._path:
             server_task = asyncio.create_task(
                 run_unix_server(self._handle_connection, self._path))
@@ -306,7 +346,7 @@ class SocketServer(EnforceOverrides):
                 except asyncio.QueueEmpty:
                     if self.to_shutdown:
                         return
-                    await asyncio.sleep(0.0023)
+                    await asyncio.sleep(0.0012)
                     continue
                 try:
                     z = await t
@@ -339,6 +379,10 @@ def make_server(app: SocketApplication, **kwargs):
 
 
 def run_app(app, **kwargs):
+    '''
+    End user typicall calls this function to start the server.
+    The server will stay up until the client requests its shutdown.
+    '''
     server = make_server(app, **kwargs)
     asyncio.run(server.run())
 
@@ -355,6 +399,10 @@ class SocketClient(EnforceOverrides):
         '''
         `path`, `host`, `port`: either `path` is given (for Unix socket),
             or `port` (plus optionally `host`) is given (for Tcp socket).
+            These values should, of course, be consistent with the corresponding
+            server.
+        `num_connections`: this is expected to have a direct impact on the performance,
+            hence needs experimentations.
         `connection_timeout`: how many seconds to wait while connecting to the server.
             This is meant for waiting for server to be ready, rather than for
             the action of "connecting" itself (which should be fast).
@@ -383,7 +431,7 @@ class SocketClient(EnforceOverrides):
         self._num_connections = num_connections or MAX_THREADS
         self._connection_timeout = connection_timeout
         self._backlog = backlog
-        self._encoder = 'orjson'  # encoder when sending requests.
+        self._encoder = 'pickle'  # encoder when sending requests.
         self._prepare_shutdown = threading.Event()
         self._to_shutdown = threading.Event()
         self._executor = concurrent.futures.ThreadPoolExecutor()
@@ -532,7 +580,7 @@ class SocketClient(EnforceOverrides):
         the server's response to the request. The user will not be
         able to resume the wait for the result.
 
-        If caller does not need the response, use `timeout=0`.
+        If caller does not need the response, use `response_timeout=0`.
 
         In some cases, the request does not need to send data, e.g. if the request
         if for certain info query. In such situations, the corresponding function on
@@ -559,6 +607,7 @@ class SocketClient(EnforceOverrides):
         If `return_x` is `False`, return a stream of `y`.
         If `return_exceptions` is `True`, `y` could be an Exception object.
         '''
+        # Refer to `mpserver.MPServer.stream` for in-code documentation.
         tasks = queue.Queue(self._backlog)
         nomore = object()
 
